	module global
c
c
cm    Srce         srce terms                       lbm/sec
cm    Srcex        srce terms                       lbm/sec
cm    rlink         valve(junction) position         no units
cm    p             steam line pressures              psia
cm    w             junction  flows                   lbm/sec
cm    y             dynamic admittance                1/ft
cm    Rho           nodal state density              lbm/ft3
cm    h             saturated stm enthalpy           (btu/lbm)
cm    t             temp. (degf) msh fluid node      degf
cm    Rhom          nodal mixture density            lbm/ft3
cm    Xe            equilibrium quality              no units
cm    Q             heat source/sink                 btu/sec
cm    time          system time                      sec
c
c --  constants
c
cm    cgc        grav const/acc 32.2             ft/sec2
cm    c144      psia to psft                   144 in2/ft2
cm    Area        volume area                       ft2
cm    Kk        link loss coefficent            no units
cm    Inrt        link inertia                      1/ft
cm    Inrt        inertia at junction (l/a)         1/ft
cm    Znode       node center elevations             ft
cm    dt        time step                        seconds
cm    Ak        junction area                      ft2
cm    V         node volume                        ft3
cm    zdh       link elev diff betwn nds           ft
cm    mprt       print flag                       no units
cm    mprtflg       print flag                       no units
cm    intn      # of internal nodes              no units
cm    inlinks      # of int. junctions              no units
cm    iu        link integer from    array       no units
cm    tooo        link integer to      array       no units
cm    bnode        array flag for pressure b.c.     no units
cm    nvols      total # of nodes in model        no units
cm    links       # of junctions in model         no units
cm    bw        bandwidth                        no units
c
c --  loctemp
c
cm    a          global matrix                    no units
cm    brhs       global rhs                       no units
cm    zdrodh        drho/dh                      (lbm/ft3)/(btu/lbm)
cm    zdrodp        drho/dp                      (lbm/ft3)/psia
cm    Rhok      donored link density             lbm/ft3
cm    ho       past time node enthalpy          btu/lbm
cm    po       past time node pressure          psia
cm    diag       vector for lhs calcs             no units
cm    alp       node void fractions              no units
c
	integer size,nts,twoo,ten,i,k
	parameter (size=100,twoo=2,ten=10)
	real c144,cgc
	parameter (c144=144.0,cgc=32.2)
      real Srce(size)
      real Srcex(size)
      real rlink(size)
      real p(size)
      real w(size)
      real y(size)
      real Rho(size)
	real Rhoo(size)
      real h(size)
      real t(size)
      real Rhom(size)
      real Xe(size)
      real Q(size)
      real time
	real sigma(size)
	real taurhs(size)
	real betaj(size)
	real d(size)
c
c --  constants
c
      real Area(size)
      real Kk(size)
      real Inrt(size)
      real Znode(size)
      real dt
      real Ak(size)
      real V(size)
      real zdh(size)
      integer mprt
      integer mprtflg
      integer intn
      integer inlinks
      integer from(size)
      integer tooo(size)
      integer bnode(size)
      integer nvols
      integer links
      integer hbw
      integer bw
c
c --  loctemp
c
      real a(size,size)
      real brhs(size)
      real DroDh(size)
      real DroDp(size)
      real Rhok(size)
      real ho(size)
      real po(size)
      real diag(size)
      real alp(size)
	logical jdone
	character*70 title
	character fname*16, answer*8
	logical fexist
      real zsgncp1,zsgnj,tiny,small,eps,big
	parameter (zsgncp1=.1,zsgnj=778,tiny=1.e-02,small=1e-06,eps=1.0,
     1big=1.e+20)
	end module global 
c 
	module steam
c	use global
	implicit none
	integer sizstm,units
	parameter (sizstm=50)
	real press,hvap,vsup1,vsup2,vsup3,vsup4
	real dvsup1dp,dvsup3dp,dvsup4dp
	real dvsup1dh,dvsup2dh,dvsup3dh,dvsup4dh
	real dhgdp
	real heos(sizstm),peos(sizstm),degtg(sizstm),degdhg,dFbardtg                                                                  
	real convpm2pe,convrom2roe,convj2btu,convkg2lbm,tc1,tc2
	parameter (convpm2pe=1.450377e-4,convrom2roe=6.242795e-2
     1,convj2btu=9.478171e-4,convkg2lbm=2.204622,tc1=1.8,tc2=459.67)
	real nuf,nug,dnudh,dnugdp,dnufdp,dnudp
	real dtl,dFbarde,deg,drgdhg,betaa,dtg
	real delsatdp,drldelsat,drlsatdp,Fbar,droldh(sizstm)
	real gs,cps,dcpsdp,dgsdp,x,xx,xx2,xxx,rk,rolsat(sizstm)
	real elsat(sizstm)
	real drkdp,dbetdp,yy,drgdeg,zz
	real alpha(sizstm),tg(sizstm),tl(sizstm),eg(sizstm),el(sizstm),
     1rog(sizstm),rol(sizstm),hl(sizstm),hg(sizstm),hsg(sizstm),
     2hsl(sizstm),
     3tsat(sizstm),hgs(sizstm),dhgsdp(sizstm),es(sizstm),desdp(sizstm)      
      real droldp(sizstm),drogdp(sizstm),drldtl(sizstm),drgdtg(sizstm),
     1deldp(sizstm),degdp(sizstm),deldtl(sizstm),degdtg(sizstm),
     2dhsldp(sizstm),dhsgdp(sizstm),dtsdp(sizstm)
	real zero,pt5,one,two,three,twoe6,tl0
	real four,five
	real c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,
     +c20,c21,c23,c24,c26,c45,c47,c48,c49,c51,c28,c40,c41,c42,c43,c44
	real a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16,a17,
     +a18
c	real c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,
c     +c24,c26,c45,c47,c48,c49,c51
c	real a1,a2,a3,a4,a5,a6,a11,a12,a13,a14,a15,a16,a18
	real sl0,sl1,sl2,sl3
	real sh0,sh1,sh2,sh3
	real hg0,hg1,hg2,hg3,hg4
	real hl0,hl1,hl2,hl3,hl4,hl5
      real cnr2(12)
      data cnr2/-.1403086182e+04,.1802594763e+01,-.2097279215e-03,       
     + .3817195017e+00,-.5394444747e-03,.1855203702e-06,                
     + -.6449501159e-04,.8437637660e-07,-.2713755001e-10,               
     + .7823817858e-08,-.1053834646e-10,.3629590764e-14/
      data zero,pt5,one,two,three,twoe6,tl0/0.0,0.5,1.0,2.0,3.0,2.0e+06
     +,573.15/
      data four,five/4.0,5.0/
      data c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,
     +c20,c21,c23,c24,c26,c28,c40,c41,c42,c43,c44,
     +c45,c47,c48,c49,c51/117.8,0.223,255.2,958.75
     +,-0.8566,2619410.618,-4.995e+10,3.403e+05,1.0665545,1.02e-08,
     +-2.548e-15,2589600.0,6.350e-03,-1.0582e-09,1.0764,3.625e-10,
     +-9.063e-17,461.7,2e6,647.3,1.3,0.3,1e5,273,239.36,2.7867,-5.77626,
     +3.938,1.0e-06,1000.0,-0.15e+03,-20.0,0.657e-06/
c      data c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,
c     +c24,c26,c45,c47,c48,c49,c51/117.8,0.223,255.2,958.75,-0.8566,
c     +2619410.618,-4.995e+10,3.403e+05,1.0665545,1.02e-08,-2.548e-15,
c     +2589600.0,6.350e-03,-1.0582e-09,1.0764,3.625e-10,-9.063e-17,
c     +1.3,0.3,1.0e-06,1000.0,-0.15e+03,-20.0,0.657e-06/
c      data a1,a2,a3,a4,a5,a6,a11,a12,a13,a14,a15,a16,a18/2.6269e-04,
c     +-0.777,1.2687552,-1.8566,-2.0e-05,-3.0e-04,1.2958e-03,593.64796,
c     +1.6845e-03,1.0e-05,1.5448e-03,-5.096e-15,-1.8126e-16/
      data sl0,sl1,sl2,sl3/-1.4655677e+06,6.926955e+03,-7.7423067,
     +7.2803006e-03/
      data sh0,sh1,sh2,sh3/-8.9,2.3639439e+04,-77.434017,7.0215574e-02/
      data hg0,hg1,hg2,hg3,hg4/2.739623397e+06,3.75884454e-02,
     1     -7.163990945e-09,4.200231947e-16,-9.850752122e-24/
      data hl0,hl1,hl2,hl3,hl4,hl5/5.747471844e+05,2.092062386e-01,
     1     -2.805106994e-08,2.380982843e-15,-1.004266028e-22,
     2      1.658695956e-30/
	end module steam
c    
	program runcv
	use global
       open(unit=10,file='cv.out',status='old')
        call cvi
         jdone=.false.
         time = 0.
            do while(.not.jdone)
          write(*,*) ' '
          write(*,*) ' input # time steps , dt , mprt  '
          read(*,*) nts,dt,mprt
           if(nts.gt.0) then
          do i=1,nts
          time = time + dt
          call cv
          end do
          else
         jdone=.true.
          endif
            end do
      stop
      end 
c
      subroutine cvi
	use global
	use steam
	call rdfn
	call initzero
	call readdata
	call elev
	call chk4zerov
c	call time0props
	call time0thermo
	call output
      return
      end
c
      subroutine cv
	use global
	use steam
	call setem
c      call dprops
	call derivthermo
	call init
	call pressure
      call gauss
	call getp
	call flow
	call enth
c	call eos
	call eosthermo
	call mass
      call courout
      return
      end
c
c/////////////////////////////////////////////////////////////////////c
c//////////////////////  gaussian elimination  ///////////////////////c
c/////////////////////////////////////////////////////////////////////c
c
c
      subroutine gauss
	use global
c
c      dimension d(40)
c
c solution of linear systems of equations
c by the gauss elimination method
c brebbia, pg 53
c
c amat = system matrix
c brhs = rhs column vector
c brhs contains the solution vector after elimination
c
c                                                   
c --  set the parameters                                                        
c                                                                               
      nd = nvols
	n1 = nd-1
      do 100 k =1,n1
      c= a(k,k)
      k1 = k+ 1
      if(abs(c) -.0000001) 101,101,301
c
c divine row by diagonal coefficent
c
301   do 105 j = k1,nd
      a(k,j) = a(k,j)/c
105   continue
      brhs(k) = brhs(k)/c
c
c eliminate unknown x(k) from row i
c
      do 108 i =k1,nd
      c=a(i,k)
      do 107 j = k1,nd
      a(i,j) = a(i,j) -c*a(k,j)
 107  continue
      brhs(i) = brhs(i) -c*brhs(k)
 108  continue
 100  continue
      if(abs(a(nd,nd)) -.0000001) 101,101,102
c
c compute last unknown
c
 102  brhs(nd) = brhs(nd)/a(nd,nd)
c
c apply backsubstitution process to compute remaining 
c
      do 205 l=1,n1
      k = nd-l
      k1 = k+1
      do 205 j=k1,nd
 205  brhs(k) = brhs(k)-a(k,j)*brhs(j)
      go to 300
c
101   write(*,*) ' singularity for row',k
      stop
c
c
 300  continue
c
      return
      end	
c
c
c///////////////////////////////////////////////////////////////////////
c//////////////           deriv props          /////////////////////////
c///////////////////////////////////////////////////////////////////////
c
      subroutine dprops
	use global
c
c -- get the derivatives at all nodes since the rel. flux
c -- terms in the energy eq. are present
c
      do 555 m=1,nvols
c
c -- get the mix density derivative per equation sgms-4a
c
c -- zdrgdp   = -(1/nu)*(1/nu)*(dnu/dp)
c -- zdrgdp1  = derivative for node n1
c -- zdrgdp2  = derivative for node n2
c -- znug1    = specific volume for node n1
c -- znug2    = specific volume for node n2
c
c
c -- the derivative of specific volume is done numerically
c -- per equation sgms-4b
c
c -- the following logic avoids the property calls for boundary nodes
c -- since they are the off diagonal terms
c
c
c --  calculate the mixture specific volume, set pressures, quality
c --  enthalpy derivative first (zdrgdh1,2) , for n1,n2
c --  pressure second (zdrgdp1,2) , for n1,n2
c
      if(Xe(m).ge.1.0) go to 551
c
      if(Xe(m).gt.0.0) then
c
c
c --  call the derivative function for two phase for dr/dp, dr/dh
c
c --  the dv/dp uses the dv/dh derivative, which is why the
c --  two are done together
c
c --  dv/dh , derivative of specifc volume wrt enthalpy
c
      drhomdh = (svsvsp(p(m)) - svwvsp(p(m)))
      drhomdhh = drhomdh/(hsvsp(p(m)) - hwvsp(p(m)))
      drhomdh = -Rho(m)*Rho(m)*drhomdhh
c
c --  do the pressure derivative
c
c///////////////////////////////////////////////////////////////////////
c///////////////////  dv/dp function ///////////////////////////////////
c///////////////////////////////////////////////////////////////////////
c
c
c --  dv/dp , derivative of mixture specific volume wrt pressure
c
c --  calculate dvgdp
c
      dsgndpd  = 1.
      zsgnp1 = p(m) + dsgndpd
      dsgndp = Xe(m)*(svsvsp(zsgnp1) - svsvsp(p(m)))/dsgndpd
      dsgndp = dsgndp + (1.-Xe(m))*
     &(svwvsp(zsgnp1) - svwvsp(p(m)))/dsgndpd
c
c --  get dh/dp , derivative of mixture enthalpy wrt pressure
c
      dsgnhdp = Xe(m)*(hsvsp(zsgnp1) - hsvsp(p(m)))/dsgndpd
      dsgnhdp = dsgnhdp + (1. - Xe(m))*
     &(hwvsp(zsgnp1) - hwvsp(p(m)))/dsgndpd
c
c --  get dvm/dp , partial derivative of mixture spec volume wrt pressur
c --  remember that dv/dh is used here
c
      drhomdp = dsgndp - drhomdhh*dsgnhdp
      drhomdp = -Rho(m)*Rho(m)*drhomdp
c
c
c
      DroDh(m) = drhomdh
      DroDp(m) = drhomdp
c
      else
c
c
c --  deriv of liq density wrt enthalpy
c
      hvalr = h(m) + 1.
      rholr = extvvwhp(hvalr,p(m))
      rholr = 1./rholr
      rholl = extvvwhp(h(m),p(m))
      rholl = 1./rholl
      drholdh = rholr - rholl
c
c
c --  deriv of liq density wrt pressure
c
      pvalr = p(m) + 1.
      rholr1 = extvvwhp(h(m),pvalr)
      rholr1 = 1. / rholr1
c     rholl = extvvwhp(h,p)
c     rholl = 1./rholl
      drholdp = rholr1 - rholl
c
      DroDh(m) = drholdh
      DroDp(m) = drholdp
      endif
c
c
      go to 555
c
 551  continue
c
c/////////////////////////////////////////////////////////////////////c
c//////////// superheated enthalpy derivative drhogdh ////////////////c
c/////////////////////////////////////////////////////////////////////c
c
c --  deriv of gas density wrt enthalpy
c
      hvalr = h(m) + 1.
      rhogr = extvvvhp(hvalr,p(m))
      rhogr = 1./rhogr
      rhogl = extvvvhp(h(m),p(m))
      rhogl = 1./rhogl
      drhogdh = rhogr - rhogl
c
c
c/////////////////////////////////////////////////////////////////////c
c//////////// superheated pressure derivative drhogdp ////////////////c
c/////////////////////////////////////////////////////////////////////c
c
c --  deriv of gas density wrt pressure
c
      pvalr = p(m) + 1.
      rhogr = extvvvhp(h(m),pvalr)
      rhogr = 1./rhogr
c     rhogl = extvvvhp(h,p)
c     rhogl = 1./rhogl
      drhogdp = rhogr - rhogl
c
      DroDh(m) = drhogdh
      DroDp(m) = drhogdp
c
c
c
 555  continue
c
      return
      end
c
c/////////////////////////////////////////////////////////////////////c
c////////////////////  courant calcs & output ////////////////////////c
c/////////////////////////////////////////////////////////////////////c
c
      subroutine courout
	use global
c
c
      dimension zsgncnt(40)
c
c
c --  perform the courant limit calcs
c --  the courant lim = dx/v= a dx rho/a v rho
c --  = mass/w, 1/2 of v(n1)*rho(n1) & 1/2 of v(n2)*rho(n2)
c --  is the mass associated with a link , w sub j is the
c --  link flow
c
      do 110 i = 1,links
       nu = from(i)
       nd = tooo(i)
      zsgncnt(i) = .5*(V(nu)*Rho(nu) + V(nd)*Rho(nd))
      if(w(i).eq.0.) ww = .01
      if(w(i).ne.0.) ww = w(i)
      zsgncnt(i) = zsgncnt(i)/ww
      zsgncnt(i) = abs(zsgncnt(i))
      if(zsgncnt(i).gt.100.) zsgncnt(i) = 100.
110   continue
c
c***********************************************************************
c*----------------------------------------------------------------------
c                           output routine
c*----------------------------------------------------------------------
c***********************************************************************
c
      mprtflg = mprtflg + 1
      if(mprtflg.ne.mprt) go to 6002
c
      write(10,*) ' '
      write(10,*) '                             time(seconds)  '
      write(10,*) '                            ',time
      write(*,*) '                             time(seconds)'
      write(*,*) '                            ',time
      write(*,*) ' '
      write(10,*) ' '
      write(10,90)
      write(*,90)
      write(10,*) ' '
      write(*,*) '  '
      do 120 i = 1,nvols
      pr = p(i)
      dc = Rho(i)
      dm = Rhom(i)
      hm = h(i)
      xq = Xe(i)
      aa = alp(i)
      qs = Q(i)
      tt = t(i)
      write(10,89) i,pr,dc,dm,hm,xq,aa,qs,tt
      write(*,89) i,pr,dc,dm,hm,xq,aa,DroDp(i),DroDh(i)
120   continue
c
      write(10,*) ' '
      write(*,*) ' '
      write(*,*) '(jun.)  w(lbm/s)    courant    '
      write(10,*) '(jun.)  w(lbm/s)    courant    '
      write(10,*) ' '
      write(*,*) '  '
      do 200  i = 1,links
      write(10,87) i,w(i),zsgncnt(i)
      write(*,87) i,w(i),zsgncnt(i)
200   continue
      write(10,*) '  '
      write(*,*) '  '
c
c
c
87    format(i2,6x,f10.2,6x,f10.4)
89    format(i2,1x,3(f7.2),1x,3(f7.2),f12.8,1x,f12.8)
90    format(' n    p      dc      dm   enth      x     alpha      qs
     &  t    ')
c
c
      mprtflg = 0
6002  continue
c
       return
       end
c
c         program myprops
c         print *,'input pressure,enthalpy'
c         read(*,*) p,h
c         hl = hwvsp(p)
c         print *,' hwvsp = ',hl
c         hv = hsvsp(p)
c         print *,' hsvsp = ',hv
c         vl = svwvsp(p)
c         print *,'svwvsp = ',vl
c         vv = svsvsp(p)
c         print *,'svsvsp = ',vv
c         tsat = tvsp(p)
c         print *,'tvsp = ',tsat
c          densl = rhol(h,p)
c          print *,' rhol = ',densl
c          droldp = drholdp(p)
c          print *,'droldp = ',droldp
c          droldh = drholdh(h)
c          print *,'droldh = ',droldh
c
c          end
c**************************************************************************c
c**************************  tvsp  ****************************************c
c**************************************************************************c
c
         function tvsp(p)
c
c        this function routine calculates sat temp
c        of water given the local pressure
c        input:  pressure (psia)
c        output: temp (degf)
c        get the sat tempc
c
         data c1t,c2t,c3t,cpp1/212.0,6.89655e-2,0.223,1./
c
c
c
c
c ---    limit tsat to 1 psia
c ---    dsl 12/18/90
c
         p = max(cpp1,p)
c
         tvsp = c1t*(c2t*p)**c3t
         end
c
c**************************************************************************c
c**************************  hwvsp  ***************************************c
c**************************************************************************c
c
         function hwvsp(p)
c
c        this function routine calculates the liq enthalpy
c        of water given the local pressure
c        input:  pressure (psia)
c        output: sat enthalpy btu/lbm
c        get the sat temp, liq int energy, liq density , compute enthalpy
c
         data c1t,c2t,c3t,cpp1/212.0,6.89655e-2,0.223,1./
         data c1,c2,c3/0.5555,255.224,573.15/
         data sl0,sl1,sl2,sl3/-6.3e+2,2.977898,-3.3284e-3,
     1        3.1298e-6/
         data sh0,sh1,sh2,sh3/-3.82611e-3,10.1626,-3.32889e-2,
     1        3.01856e-5/
         data  c1e,c2e,c3e,c4e/62.42,-5.066e-5,-2.904e-3,2.828e-4/
c
c
c
c
c ---    limit tsat to 1 psia
c ---    dsl 12/18/90
c
         p = max(cpp1,p)
c
         tsat = c1t*(c2t*p)**c3t
c
c        this function routine computes the liquid internal
c        energy given the liquid temp.
c
c        input:  liquid temp (f)
c        output: liquid internal energy (btu/lbm)
c
         tcal = c1*tsat + c2
         if(tcal .lt. c3) then
           eliq = sl0+sl1*tcal+sl2*tcal*tcal+sl3*tcal*tcal*tcal
         else
           eliq = sh0+sh1*tcal+sh2*tcal*tcal+sh3*tcal*tcal*tcal
         end if
c
c        this function routine computes the liquid density given
c        the liq internal energy and pressure.
c
c        input:  liquid internal energy (f)
c                pressure (psia)
c        output: liquid density (lbm/ft3)
c
c
         rhol = c1e + c2e*eliq*eliq + c3e*eliq + c4e*p
c
c --     output liq enthalpy
c
         hwvsp = eliq + (p/rhol)*0.18504
c
         end
c
c**************************************************************************c
c**************************  hsvsp  ***************************************c
c**************************************************************************c
c
         function hsvsp(p)
c
c
c        this function routine calculates the vapor enthalpy
c        of water given the local pressure
c        input:  pressure (psia)
c        output: sat enthalpy btu/lbm
c        get the sat temp, vap int energy, vap density , compute enthalpy
c
         data c1t,c2t,c3t,cpp1/212.0,6.89655e-2,0.223,1./
         data cts1,cts2,cts3/212.,6.89655e-2,0.223/
         data cps1,cps2,cps3,cps4/0.22899,0.6057,-8.582e-4,-0.8566/
         data ces1,ces2,ces3,ces4,ces5,ces6,ces7/1.126e+3,-2.147e+7,
     1        3.403e+5,6.8966e+3,1.1132e+3,-2.1637e-5,1.8827e-2/
         data cb1,ceg1,cpp2/5.4256,0.14178,3180/
         data cg1,cg2,cg3,cg4,cg5,cg6/1.0665545,7.0345e-5,
     1        -1.2119e-7,1.0764,2.5e-6,-4.3106e-9/
         data crog1,crog2/1.8507e-1,0.3/
c
c
c ---    limit tsat to 1 psia
c ---    dsl 12/18/90
c
         p = min(cpp2,max(cpp1,p))
c
         tsat = c1t*(c2t*p)**c3t
c
c        this routine computes the internal energy of
c        vapor given the vapor temp and pressure.
c
c        input:   vapor temp (f)
c                 pressure (psia)
c        output:  vapor internal energy (btu/lbm)
c
c
         cps = cps1*(cps2+cps3*tsat)**cps4
         if(p.lt.290.0) then
            es = ces1 + ces2*(1.0/(ces3+ces4*p))
         else
            es = ces5 + ces6*p*p + ces7*p
         end if
         beta = tsat*tsat*(1.0-1.0/(cb1*cps-1.0)**2)
         evap = es + ceg1*((tsat-tsat)+sqrt(tsat*tsat-beta) -
     1          tsat/(cb1*cps-1.0))
c
c
c        this routine computes the vapor density given the 
c        vapor internal energy and pressure.
c
c        input:   vapor internal energy (btu/lbm)
c                 pressure (psia)
c        output:  vapor density (lbm/ft3)
c
c
         if(p.lt.290.0) then
           gs = cg1 + cg3*p*p + cg2*p
         else
           gs = cg4 + cg6*p*p + cg5*p
         end if
c
         rhog = crog1*p/((gs-1.0)*evap)
c
c --     compute the vapor enthalpy
c
         hsvsp = evap + (p/rhog)*0.18504
c
         end
c
c**************************************************************************c
c**************************  swsvsp  **************************************c
c**************************************************************************c
c
c
         function svwvsp(p)
c
c        this function routine calculates the liq sp v olume
c        of water given the local pressure
c        input:  pressure (psia)
c        output: sat liq sp vol ft3/lbm
c        get the sat temp, liq density , compute sp vol
c
         data c1t,c2t,c3t,cpp1/212.0,6.89655e-2,0.223,1./
         data c1,c2,c3/0.5555,255.224,573.15/
         data sl0,sl1,sl2,sl3/-6.3e+2,2.977898,-3.3284e-3,
     1        3.1298e-6/
         data sh0,sh1,sh2,sh3/-3.82611e-3,10.1626,-3.32889e-2,
     1        3.01856e-5/
         data  c1e,c2e,c3e,c4e/62.42,-5.066e-5,-2.904e-3,2.828e-4/
c
c
c
c
c ---    limit tsat to 1 psia
c ---    dsl 12/18/90
c
         p = max(cpp1,p)
c
         tsat = c1t*(c2t*p)**c3t
c
c        this function routine computes the liquid internal
c        energy given the liquid temp.
c
c        input:  liquid temp (f)
c        output: liquid internal energy (btu/lbm)
c
         tcal = c1*tsat + c2
         if(tcal .lt. c3) then
           eliq = sl0+sl1*tcal+sl2*tcal*tcal+sl3*tcal*tcal*tcal
         else
           eliq = sh0+sh1*tcal+sh2*tcal*tcal+sh3*tcal*tcal*tcal
         end if
c
c        this function routine computes the liquid density given
c        the liq internal energy and pressure.
c
c        input:  liquid internal energy (f)
c                pressure (psia)
c        output: liquid density (lbm/ft3)
c
c
         rhol = c1e + c2e*eliq*eliq + c3e*eliq + c4e*p
c
         svwvsp = 1/rhol
c
         end
c
c**************************************************************************c
c**************************  svsvsp  **************************************c
c**************************************************************************c
c
         function svsvsp(p)
c
c
c        this function routine calculates the vapor sp vol
c        of water given the local pressure
c        input:  pressure (psia)
c        output: sat vap sp vol ft3/lbm
c        get the sat temp, vap int energy, vap density , compute sp vol
c
         data c1t,c2t,c3t,cpp1/212.0,6.89655e-2,0.223,1./
         data cts1,cts2,cts3/212.,6.89655e-2,0.223/
         data cps1,cps2,cps3,cps4/0.22899,0.6057,-8.582e-4,-0.8566/
         data ces1,ces2,ces3,ces4,ces5,ces6,ces7/1.126e+3,-2.147e+7,
     1        3.403e+5,6.8966e+3,1.1132e+3,-2.1637e-5,1.8827e-2/
         data cb1,ceg1,cpp2/5.4256,0.14178,3180/
         data cg1,cg2,cg3,cg4,cg5,cg6/1.0665545,7.0345e-5,
     1        -1.2119e-7,1.0764,2.5e-6,-4.3106e-9/
         data crog1,crog2/1.8507e-1,0.3/
c
c
c ---    limit tsat to 1 psia
c ---    dsl 12/18/90
c
         p = min(cpp2,max(cpp1,p))
c
         tsat = c1t*(c2t*p)**c3t
c
c        this routine computes the internal energy of
c        vapor given the vapor temp and pressure.
c
c        input:   vapor temp (f)
c                 pressure (psia)
c        output:  vapor internal energy (btu/lbm)
c
c
         cps = cps1*(cps2+cps3*tsat)**cps4
         if(p.lt.290.0) then
            es = ces1 + ces2*(1.0/(ces3+ces4*p))
         else
            es = ces5 + ces6*p*p + ces7*p
         end if
         beta = tsat*tsat*(1.0-1.0/(cb1*cps-1.0)**2)
         evap = es + ceg1*((tsat-tsat)+sqrt(tsat*tsat-beta) -
     1          tsat/(cb1*cps-1.0))
c
c
c        this routine computes the vapor density given the 
c        vapor internal energy and pressure.
c
c        input:   vapor internal energy (btu/lbm)
c                 pressure (psia)
c        output:  vapor density (lbm/ft3)
c
c
         if(p.lt.290.0) then
           gs = cg1 + cg3*p*p + cg2*p
         else
           gs = cg4 + cg6*p*p + cg5*p
         end if
c
         rhog = crog1*p/((gs-1.0)*evap)
c
         svsvsp = 1/rhog
c
         end
         function extvvwhp(hliq,press)
c
c        this function routine computes the liquid sp. volume given
c        the liq enthalpy and pressure.
c        based on trac props for internal energy, will be adjusted
c        for enthalpy
c
c        input:  liquid enthalpy (btu/lbm)
c                pressure (psia)
c        output: liquid sp. volume (ft3/lbm)
c
         data  c1,c2,c3,c4/62.42,-5.066e-5,-2.904e-3,2.828e-4/
c
         rhol = c1 + c2*hliq*hliq + c3*hliq + c4*press
         extvvwhp = 1/rhol
         end
c
c************************************************************************
         function  extvvvhp(hvap,press)
c************************************************************************
         real hvap,h
         dimension cn2(12)
c
c        this function computes the vapor sp. volume given the 
c        vapor enthalpy and pressure.
c
c        input:   vapor enthalpy (btu/lbm)
c                 pressure (psia)
c        output:  vapor sp. vol (ft3/lbm)
c        ref --   trac-p1a nureg/cr-0063
c
c                                                                     
c  cn2(i)                                                               
c                                                                       
      data cn2/-.1403086182e+04,.1802594763e+01,-.2097279215e-03,       
     + .3817195017e+00,-.5394444747e-03,.1855203702e-06,                
     + -.6449501159e-04,.8437637660e-07,-.2713755001e-10,               
     + .7823817858e-08,-.1053834646e-10,.3629590764e-14/ 
c
      p = press
      h = hvap
c
      vsup1=(cn2(1)+(cn2(2)+cn2(3)*h)*h)/p               
      vsup2=cn2(4)+(cn2(5)+cn2(6)*h)*h                     
      vsup3=cn2(7)+(cn2(8)+cn2(9)*h)*h                        
      vsup4=cn2(10)+(cn2(11)+cn2(12)*h)*h                  
      watpr=vsup1+vsup2+(vsup3+vsup4*p)*p              
         extvvvhp = watpr         
         return   
         end
c*********************************************************************
c**************************          *********************************
c*********************************************************************
      function extttwhp(p,h)
c
      dimension ct1(8)
c                                                                       
c  ct1(i)                                                               
c
      data ct1/.3276275552e+02,.3360880214e-02,                         
     + .9763617e+00,-.559528176e-04,.1857226027e-03,                   
     + .1618595991e-06,-.468267433e-06,-.1180204381e-09/
c
c --  extttwhp is the subcooled liquid temp 
c --  as a function of pressure and liquid enthalpy
c                                                                       
c subcooled properties                                                  
c                                                                       
c
      ct1f1=ct1(3)+ct1(4)*p                                            
      ct1f2=ct1(5)+ct1(6)*p                                            
      ct1f3=ct1(7)+ct1(8)*p                                           
      watpr=ct1(1)+ct1(2)*p+(ct1f1+(ct1f2+ct1f3*h)*h)*h  
c                                                        
c
c --  calc the subcooled liq temp 
c
      extttwhp = watpr
      return
      end
c*********************************************************************
c**************************          *********************************
c*********************************************************************
      function extttvhp(p,h)
c
      dimension ct3(25)
c                                                                       
c  ct3(i)                                                               
c                                                                       
      data ct3/-.1179100862e+05,.1256160907e+03,                        
     + -.1083713369e+00,.3278071846e-04,-.3425564927e-08,              
     + .2829274345e+02,-.3333448850e+00,.2928177730e-03,                
     + -.8970959364e-07,.9527692453e-11,-.2678181564e-01,               
     + .3326901268e-03,-.2972436458e-06,.9246248312e-10,                
     + -.1001409043e-13,.1218742752e-04,-.1477890326e-06,               
     + .1342639113e-09,-.4249155515e-13,.4703914404e-17,                
     + -.2092033147e-08,.2463258371e-10,-.2275585718e-13,               
     + .7338316751e-17,-.8315044742e-21/ 
c
c --  extttvhp is the superheated vapor temp 
c --  as a function of pressure and vapor enthalpy
c --  calculate the superheated vapor temp
c                                                                       
c superheat properties                                                  
c                                                                       
c     temperature  as a function of pressure and enthalpy               
c     entry tsup
c                                                        
      ct3f1=ct3(1)+(ct3(2)+(ct3(3)+(ct3(4)+ct3(5)*p)*p)*p)*p   
      ct3f2=ct3(6)+(ct3(7)+(ct3(8)+(ct3(9)+ct3(10)*p)*p)*p)*p    
      ct3f3=ct3(11)+(ct3(12)+(ct3(13)+(ct3(14)+ct3(15)*p)*p)*p)*p 
      ct3f4=ct3(16)+(ct3(17)+(ct3(18)+(ct3(19)+ct3(20)*p)*p)*p)*p
      ct3f5=ct3(21)+(ct3(22)+(ct3(23)+(ct3(24)+ct3(25)*p)*p)*p)*p  
      watpr=ct3f1+(ct3f2+(ct3f3+(ct3f4+ct3f5*h)*h)*h)*h         
c
c --  calc the sh vapor temp 
c
      extttvhp = watpr
      return
      end
      function tvshw(h3)
c /home/ittc/fitch/testbed/dp_incl *!dbr
c-----tabelldata
c
      real h3
c /home/ittc/fitch/testbed/dp_incl *!dbr
      dimension Ak(5), ya(4), aa(4), ba(4), ca(4)
c-----konvertera fr n kcal/kg till btu/lb
c
      data Ak / 8., 68., 575.6, 823.4, 905.4 /
      data ya / 40., 100., 570., 700. /
      data aa / 1., .92593, .52462, .06687 /
      data ba / 4.3824e-4, -1.4485e-4, -1.7334e-5, -3.7790e-3 /
      data ca / -3.4237e-6, -3.2951e-7, -1.4779e-6, 2.9376e-6 /
      data cproppme / 1.0 /
      data cprophem / 1.0 /
      data ccon32 / 0. /
      data cproppem / 1.0 /
      data cprophme / 1.0 /
      data cpropvem / 1.0 /
      z = cprophme * h3
c-----arg < x(3)
c
      if (z .gt. Ak(3)) goto 1050
      if (z .gt. Ak(2)) then
      ir0 = 2
      goto 1099
      end if
      ir0 = 1
      goto 1099
c-----arg > x(3)
c
 1050 continue
      if (z .gt. Ak(4)) then
      ir0 = 4
      goto 1099
      end if
      ir0 = 3
 1099 continue
c-----konvertera fr n grad-f till grad-c
c
      propeq = ya(ir0) + ((Ak(ir0) - z) * ((((ca(ir0) * z) + ba(ir0)) *
     &(Ak(ir0 + 1) - z)) - aa(ir0)))
      tvshw = cprophem * (propeq - ccon32)
      return
      end
c
c --
c
      function pvshw(h1)
c /home/ittc/fitch/testbed/dp_incl *!dbr
c-----tabelldata
c
      real h1
c /home/ittc/fitch/testbed/dp_incl *!dbr
      dimension x5(10), y5(9), a5(9), b5(9), c5(9)
c-----konvertera fr n kcal/kg till btu/lb
c
c-----s k r tt intervall f r argumentet
c
      data x5 / 8., 28.1, 68., 112.9, 173.1, 368.6, 512., 679.1, 823.4,
     &905.2 /
      data y5 / .12168, .25621, .95612, 3.2815, 12.770, 230.83, 812.42,
     &2060.3, 3093.8 /
      data a5 / .006693, .017542, .05179, .15762, 1.1154, 4.0557, 7.4679
     &, 7.1622, 1.3985 /
      data b5 / 1.1211e-4, 1.6992e-4, 2.9634e-4, 4.6278e-4, 2.2996e-3,
     &7.0736e-3, .020869, .039889, -.062001 /
      data c5 / 7.3987e-7, 1.8776e-6, 3.2975e-6, 6.7154e-6, 1.2060e-5,
     &9.0109e-6, -2.2875e-5, -7.1759e-5, 4.3755e-5 /
      data cproppme / 1.0 /
      data cprophem / 1.0 /
      data ccon32 / 0. /
      data cproppem / 1.0 /
      data cprophme / 1.0 /
      data cpropvem / 1.0 /
      z = cprophme * h1
c-----arg < x(6)
c
      if (z .gt. x5(6)) goto 560
c-----arg < x(4)
c
      if (z .gt. x5(4)) goto 540
c-----arg < x(2)
c
      if (z .gt. x5(2)) goto 530
      ir0 = 1
      goto 599
c-----x(2) < arg < x(4)
c
  530 continue
      if (z .gt. x5(3)) then
      ir0 = 3
      goto 599
      end if
      ir0 = 2
      goto 599
c-----x(4) < arg < x(6)
c
  540 continue
      if (z .gt. x5(5)) then
      ir0 = 5
      goto 599
      end if
      ir0 = 4
      goto 599
c-----arg > x(6)
c
  560 continue
c-----x(6) < arg < x(8)
c
      if (z .gt. x5(8)) goto 580
      if (z .gt. x5(7)) then
      ir0 = 7
      goto 599
      end if
      ir0 = 6
      goto 599
c-----arg > x(8)
c
  580 continue
      if (z .gt. x5(9)) then
      ir0 = 9
      goto 599
      end if
      ir0 = 8
  599 continue
c-----konvertera fr n psi till bar
c
      propeq = y5(ir0) + ((x5(ir0) - z) * ((((c5(ir0) * z) + b5(ir0)) *
     &(x5(ir0 + 1) - z)) - a5(ir0)))
      pvshw = cproppem * propeq
      return
      end
c
c --
c
      function pvst(t3)
c /home/ittc/fitch/testbed/dp_incl *!dbr
c-----tabelldata
c
      real t3
c /home/ittc/fitch/testbed/dp_incl *!dbr
      dimension x6(8), y6(7), a6(7), b6(7), c6(7)
c-----konvertera fr n grad-c till grad-f
c
c-----ber kning sker enligt 1/p = f(1/t)
c
c-----s k r tt intervall f r argumentet
c
      data x6 / .025000, .016667, .0083333, .0057143, .0037736, .0024510
     &, .0017544, .0014175 /
      data y6 / 8.2183, 3.9030, .59081, .14890, .025954, .0036951,
     &.00081520 /
      data a6 / 517.83, 397.47, 168.73, 63.351, 16.829, 4.1342, 1.4945 /
      data b6 / 8284.5, 25603., 22368., 9909.4, 2670.1, 578.58, -314.4 /
      data c6 / -268720., -782320., 295410., 1746900., 2048800.,
     &1399000., 1134600. /
      data cproppme / 1.0 /
      data cprophem / 1.0 /
      data ccon32 / 0. /
      data cproppem / 1.0 /
      data cprophme / 1.0 /
      data cpropvem / 1.0 /
      z = 1. / ((cprophme * t3) + ccon32)
c-----arg < x(5)
c
      if (z .lt. x6(5)) goto 650
c-----arg < x(3)
c
      if (z .lt. x6(3)) goto 640
      if (z .lt. x6(2)) then
      ir0 = 2
      goto 699
      end if
      ir0 = 1
      goto 699
c-----x(3) < arg < x(5)
c
  640 continue
      if (z .lt. x6(4)) then
      ir0 = 4
      goto 699
      end if
      ir0 = 3
      goto 699
c-----arg > x(5)
c
  650 continue
      if (z .lt. x6(7)) then
      ir0 = 7
c-----x(5) < arg < x(7)
c
      goto 699
      end if
      if (z .lt. x6(6)) then
      ir0 = 6
      goto 699
      end if
      ir0 = 5
  699 continue
c-----konvertera fr n psi till bar
c
      propeq = y6(ir0) + ((x6(ir0) - z) * ((((c6(ir0) * z) + b6(ir0)) *
     &(x6(ir0 + 1) - z)) - a6(ir0)))
      pvst = cproppem / propeq
      return
      end
c
c --
c
      function swvvsh(h2)
c /home/ittc/fitch/testbed/dp_incl *!dbr
c-----tabelldata
c
      real h2
c /home/ittc/fitch/testbed/dp_incl *!dbr
      dimension x7(5), y7(4), a7(4), b7(4), c7(4)
c-----konvertera fr n kcal/kg till btu/lb
c
c-----s k r tt intervall f r argumentet
c
      data x7 / 905.2, 646.9, 290.4, 98.0, 8.0 /
      data y7 / .05078, .02467, .01766, .01625 /
      data a7 / 1.0108e-4, 1.9663e-5, 7.3285e-6, 2.6667e-6 /
      data b7 / -1.5435e-7, -1.7128e-8, 1.7286e-8, -5.8844e-10 /
      data c7 / 6.5193e-10, 1.1572e-10, -8.7123e-11, 1.3882e-10 /
      data cproppme / 1.0 /
      data cprophem / 1.0 /
      data ccon32 / 0. /
      data cproppem / 1.0 /
      data cprophme / 1.0 /
      data cpropvem / 1.0 /
      z = cprophme * h2
c-----arg > x(3)
c
      if (z .lt. x7(3)) goto 750
      if (z .lt. x7(2)) then
      ir0 = 2
      goto 799
      end if
      ir0 = 1
      goto 799
c-----arg < x(3)
c
  750 continue
      if (z .lt. x7(4)) then
      ir0 = 4
      goto 799
      end if
      ir0 = 3
  799 continue
c-----konvertera fr n ft3/lb till m3/kg
c
      propeq = y7(ir0) + ((x7(ir0) - z) * ((((c7(ir0) * z) + b7(ir0)) *
     &(x7(ir0 + 1) - z)) - a7(ir0)))
      swvvsh = cpropvem * propeq
      return
      end
c
c --
c
c
c --
c
      function svwvsh(h2)
c /home/ittc/fitch/testbed/dp_incl *!dbr
c-----tabelldata
c
      real h2
c /home/ittc/fitch/testbed/dp_incl *!dbr
      dimension x7(5), y7(4), a7(4), b7(4), c7(4)
c-----konvertera fr n kcal/kg till btu/lb
c
c-----s k r tt intervall f r argumentet
c
      data x7 / 905.2, 646.9, 290.4, 98.0, 8.0 /
      data y7 / .05078, .02467, .01766, .01625 /
      data a7 / 1.0108e-4, 1.9663e-5, 7.3285e-6, 2.6667e-6 /
      data b7 / -1.5435e-7, -1.7128e-8, 1.7286e-8, -5.8844e-10 /
      data c7 / 6.5193e-10, 1.1572e-10, -8.7123e-11, 1.3882e-10 /
      data cproppme / 1.0 /
      data cprophem / 1.0 /
      data ccon32 / 0. /
      data cproppem / 1.0 /
      data cprophme / 1.0 /
      data cpropvem / 1.0 /
      z = cprophme * h2
c-----arg > x(3)
c
      if (z .lt. x7(3)) goto 750
      if (z .lt. x7(2)) then
      ir0 = 2
      goto 799
      end if
      ir0 = 1
      goto 799
c-----arg < x(3)
c
  750 continue
      if (z .lt. x7(4)) then
      ir0 = 4
      goto 799
      end if
      ir0 = 3
  799 continue
c-----konvertera fr n ft3/lb till m3/kg
c
      propeq = y7(ir0) + ((x7(ir0) - z) * ((((c7(ir0) * z) + b7(ir0)) *
     &(x7(ir0 + 1) - z)) - a7(ir0)))
      svwvsh = cpropvem * propeq
      return
      end
c
	subroutine enth
	use global
c
c/////////////////////////////////////////////////////////////////////c
c////////////////////  enthalpy solution    //////////////////////////c
c/////////////////////////////////////////////////////////////////////c
c
c
c --  solve for the enthalpy
c
c
c
c --  initialize the global matrix & rhs vectors
c --  rows -> nodes, columns -> bandwidth
c --  note that this is diagonal matrix assembly
c
c
c
c*********************************************************************
      do 39 i = 1,nvols
c
c --  rhs term
c
      brhs(i) = V(i)*Rho(i)*h(i)
     & + dt*Q(i) 
     & + c144*V(i)*(p(i)-po(i))/zsgnj
c
c --  diagonal term
c
      diag(i) = V(i)*Rho(i)
c
39    continue
c*********************************************************************
c
      do 41 i = 1,links
c
c --  get the upstream & downstream node numbers
c
      nu = from(i)
      nd = tooo(i)
c
c -- donor junction density
c
      if (w(i).ne.0.) go to 43
      beta = 0.
      go to 45
43    beta = w(i)/(abs(w(i)))
45    continue
c
c --  compute the ret of the terms
c
      diag(nd) = diag(nd) + dt*w(i)*(1. - .5*(
     &1.- beta))
      diag(nu) = diag(nu) - dt*w(i)*(1. - .5*(
     &1.+ beta))
      brhs(nd) = brhs(nd) + dt*w(i)*.5*h(nu)
     &*(1.+beta)
      brhs(nu) = brhs(nu) - dt*w(i)*.5*h(nd)
     &*(1.-beta)
41    continue
c
c --  get the enthalpy solution & update qualities
c
      do i = 1,nvols
	if(bnode(i).eq.0) then
      h(i) = brhs(i)/diag(i)
	endif
	enddo
c
	return
	end
c
	subroutine eos
	use global
c
c --  get the nodal densities
c
      do i = 1,nvols
c
       zsgnhfg  = hsvsp(p(i)) - hwvsp(p(i))
       Xe(i) = (h(i) - hwvsp(p(i)))/zsgnhfg
       if(Xe(i).lt.0.) Xe(i) = 0.
       if(Xe(i).gt.1.) Xe(i) = 1.
c
c     sat
c
		if(Xe(i).gt.0.0.and.Xe(i).lt.1.0) then
		vnug = svsvsp(p(i))
		vnuf = svwvsp(p(i))
		zsgnnum = Xe(i)*vnug +
     &	(1.-Xe(i))*vnuf
c
		Rho(i) = 1./zsgnnum
		alp(i) = Rho(i)*Xe(i)*vnug
		t(i) = tvsp(p(i))
		endif
c
c     subcooled
c
      if(Xe(i).le.0) then
      Rho(i) = extvvwhp(h(i),p(i))
      Rho(i) = 1./Rho(i)
      alp(i) = 0.0
      t(i) = extttwhp(p(i),h(i))
      endif
c
c     sh steam
c
		if(Xe(i).ge.1.0) then
c
		Rho(i) = extvvvhp(h(i),p(i))
		Rho(i) = 1./Rho(i)
		alp(i) = 1.0
		t(i) = extttvhp(p(i),h(i))
		endif
c
	enddo
      return 
	end
c
	subroutine eosthermo
	use global
	use steam
c
c --  get the nodal densities
c
      units = 1
	call thermo
      do i = 1,nvols
c
c     sat
c
		if(Xe(i).gt.0.0.and.Xe(i).lt.1.0) then
		zsgnnum = Xe(i)/rog(i) +
     &	(1.-Xe(i))/rolsat(i)
c
c	real convpm2pe,convrom2roe,convj2btu,convkg2lbm
c
		Rho(i) = convrom2roe/zsgnnum
		alp(i) = Rho(i)*Xe(i)/(rog(i)*convrom2roe)
		t(i) = tsat(i)*tc1-tc2
		endif
c
c     subcooled
c
      if(Xe(i).le.0) then
      Rho(i) = Rho(i)*convrom2roe
      alp(i) = 0.0
      t(i) = tc1*tl(i)-tc2
      endif
c
c     sh steam
c
		if(Xe(i).ge.1.0) then
c
		Rho(i) = Rho(i)*convrom2roe
		alp(i) = 1.0
		t(i) = tc1*tg(i)-tc2
		endif
c
	enddo
      return 
	end
c
	subroutine mass
	use global
c
c///////////////////////////////////////////////////////////////////////
c/////////////////// mixture mass solution /////////////////////////////
c///////////////////////////////////////////////////////////////////////
c
c
c --  get the diagonal volume contributions
c --  for the lhs/rhs of the mixture density equation
c --  note that this is diagonal matrix assembly
c
c
      do i = 1,nvols
      brhs(i) = Rho(i)*V(i) 
      diag(i) = V(i)
	enddo
c
      do i = 1,links
c
c --  get the upstream & downstream node numbers
c
      nu = from(i)
      nd = tooo(i)
c
c --  if the upstream or downstream node is a boundary node,
c --  don't add in any flow contribution
c --  doesn't matter since we only use internal nodes 
c --  for mixture density in the next step
c
      brhs(nu) = brhs(nu) - dt*w(i)
      brhs(nd) = brhs(nd) + dt*w(i)
c
	enddo
c
c --  get the density solution
c
      do i = 1,nvols
      if(bnode(i).eq.0) then
      Rhom(i) = brhs(i)/diag(i)
      else
c
c     set to state for a bc node
c 
      Rhom(i) = Rho(i)
      endif
c
      if(Rhom(i).lt.0.) Rhom(i) = Rho(i)
	enddo
c
	return
	end
c
	subroutine flow
	use global
c
c --  solve for the flow 
c
      do i = 1,links
      w(i) = y(i)*(p(from(i))-
     1p(tooo(i)))*dt*c144*cgc
     2 + y(i)*Inrt(i)*w(i)
     3 - dt*y(i)*Rhok(i)*cgc*zdh(i)
	enddo
	return
	end
c
	subroutine pressure
	use global
c
c --  get the volume contribution energy elimination terms
c
      do i = 1,nvols
      sigma(i) = V(i)*Rho(i)
      taurhs(i) =  dt*Q(i) + V(i)*Rho(i)*h(i)
	enddo
c
      do i = 1,links
c
      nd = tooo(i)
      nu = from(i)
c
c --  get beta
c
      if(w(i).eq.0.) then
      betaj(i) = 0.
      else
      betaj(i) = w(i)/(abs(w(i)))
      endif
c
      beta = betaj(i)
      sigma(nd) = sigma(nd) + dt* 
     & (w(i) - w(i)*.5*(1-beta)) 
      sigma(nu) = sigma(nu) - dt* 
     & (w(i) - w(i)*.5*(1+beta))
      taurhs(nu) = taurhs(nu) - dt*.5*w(i)*
     &(1-beta)*h(nd)
      taurhs(nd) = taurhs(nd) + dt*.5*w(i)*
     &(1+beta)*h(nu) 
	enddo
c
c --  form the global matrix entries
c --  volume contributions and then links
c --  diagonal contributions
c
      do i = 1,nvols
      a(i,i) = V(i)*DroDp(i) + c144*V(i)
     & *V(i)*DroDh(i)/(zsgnj*sigma(i))
      brhs(i) = V(i)*DroDp(i)*p(i) + V(i)
     & *DroDh(i)*h(i) - V(i)*DroDh(i)*taurhs(i)/sigma(i)
     & + eps*V(i)*(Rhom(i) - Rhoo(i))
     & + c144*V(i)*V(i)*DroDh(i)*p(i)/
     & (zsgnj*sigma(i))
	enddo
c
c --  add in link contributions
c
      do i = 1,links
c
c --  link density
c
c
      nd = tooo(i)
      nu = from(i)
c
      Rhok(i) = .5*(Rho(nu) + Rho(nd))
     & + .5*betaj(i)*(Rho(nu) - Rho(nd))
c
c --  get the y factor from the momentum equation sgms -
c
      wabs = abs(w(i))
      rsgnmom = rlink(i)
c
c --  limit r from going to zero
c
      if(rsgnmom.le.tiny) then
      y(i) = 0.
      else
      y(i) = (Inrt(i) + .5*dt*Kk(i)*wabs/
     &(Rhok(i)*rsgnmom*rsgnmom*Ak(i)*Ak(i)))
      y(i) = 1/y(i)
c
      endif
c
      d(i) = y(i)*Inrt(i)*w(i)
     & - dt*y(i)*Rhok(i)*cgc*zdh(i)
c
c
c
      a(nd,nu) = a(nd,nu) 
     & - dt*dt*cgc*c144*y(i)
      a(nu,nd) = a(nu,nd) 
     & - dt*dt*cgc*c144*y(i)
      a(nd,nd) = a(nd,nd) + dt*dt
     &*cgc*c144*y(i)
      a(nu,nu) = a(nu,nu) + dt*dt
     &*cgc*c144*y(i)
c
      brhs(nd) = brhs(nd) + dt*d(i)
      brhs(nu) = brhs(nu) - dt*d(i)
c
	enddo
c
c --  set boundary conditions
c
      do i = 1,nvols
      if(bnode(i).eq.1) then
      brhs(i) = a(i,i)*big*p(i)
      a(i,i) = a(i,i)*big
      else
      continue
      endif
	enddo
c
      return
	end
c
	subroutine getp
	use global
c
c --  recover the solution from the rhs of the solver
c --  sum over all internal nodes
c --  if the node is a boundary node , the value will not change
c --  if not, use the solver (solution) value
c
      do i = 1,nvols
      p(i) = brhs(i)
c
c --  limit the pressure to avoid steam table problems
c
      if(p(i).lt.zsgncp1) p(i) = zsgncp1
	enddo
	return
	end
c
	subroutine init
	use global
c
c --  init arrays to zero
c
      do i = 1,nvols
      sigma(i) = 0.
      brhs(i) = 0.
      taurhs(i) = 0.
      do j = 1,nvols
      a(i,j) = 0.
	enddo
	enddo
	return
	end
c
	subroutine setem
	use global
	do i = 1,nvols
      Rhoo(i) = Rho(i)
      Rho(i) = Rhom(i)
      po(i) = p(i)
      ho(i) = h(i)
	enddo
	return
	end
c
	subroutine time0props
	use global
c
c --  calculate the quality from enthalpy,pressure
c
      do i = 1,nvols
c
      zsgnhfg  = hsvsp(p(i)) - hwvsp(p(i))
      Xe(i) = (h(i) - hwvsp(p(i)))/zsgnhfg
      if(Xe(i).lt.0.) Xe(i) = 0.
      if(Xe(i).gt.1.) Xe(i) = 1.
c
      if(Xe(i).gt.0.0.and.Xe(i).lt.1.0) then
      znumix = Xe(i)*svsvsp(p(i)) + (1.-Xe(i))*
     &svwvsp(p(i))
      Rho(i) = 1./znumix
      Rhom(i) = Rho(i)
      t(i) = tvsp(p(i))
      endif
c
	if(Xe(i).le.0.0) then
      Rho(i) = extvvwhp(h(i),p(i))
      Rho(i) = 1./Rho(i)
      Rhom(i) = Rho(i)
      t(i) = extttwhp(p(i),h(i))
      endif
c
	if(Xe(i).ge.1.0) then
      Rho(i) = extvvvhp(h(i),p(i))
      Rho(i) = 1./Rho(i)
      Rhom(i) = Rho(i)
      t(i) = extttvhp(p(i),h(i))
	endif
c
	enddo


	return
	end
c
	subroutine time0thermo
	use global
	use steam
c
c --  calculate the quality from enthalpy,pressure
c
      units=1
	call thermo
c
c
	do i=1,nvols
      if(Xe(i).gt.0.0.and.Xe(i).lt.1.0) then
      Rho(i) = Rho(i)*convrom2roe
c	real convpm2pe,convrom2roe,convj2btu,convkg2lbm
      Rhom(i) = Rho(i)
      t(i) = tc1*tsat(i)-tc2
      endif
c
	if(Xe(i).le.0.0) then
      Rho(i) = Rho(i)*convrom2roe
      t(i) = tc1*tl(i)-tc2
      Rhom(i) = Rho(i)
      endif
c
	if(Xe(i).ge.1.0) then
      Rho(i) = Rho(i)*convrom2roe
      t(i) = tc1*tg(i)-tc2
      Rhom(i) = Rho(i)
	endif
	enddo
c
	return
	end
c
	subroutine output
	use global
c
c
c***********************************************************************
c*********************    output information    ************************
c***********************************************************************
c
c
      write(10,*) '                              title'
      write(10,100) title
      write(10,*) ' '
      write(10,*) ' iset =',iset
      write(10,*) ' '
100   format(a)
      write(10,*) 'mprt = ',mprt
      write(10,*) 'mprtflg = ',mprtflg
      write(10,*) ' '
      write(10,*) ' '
c
      write(10,*)' nvols   links '
      write(10,31) nvols,links
 31   format(5x,i2,7x,i2)
c
      write(10,*) ' '
      write(10,*) ' time step = ',dt
      write(10,*) ' '
      write(10,*) 'boundary condition information '
      write(10,*) ' '
      write(10,*) '  # of boundary nodes =',msgnnpb
      write(10,*) 'nd     b.c. flag'
      do j = 1,nvols
      write(10,7) j,bnode(j)
	enddo
  7   format(i2,8x,i2)
c
c
c --  loop over elemnt quantities and write out to the output file
c
      write(10,*) ' '
      write(10,*) 'el# from to    r       flow       k      el.area   i'
c
      do j = 1,links
      write(10,3) j,from(j),tooo(j),rlink(j),
     1w(j),Kk(j),Ak(j),Inrt(j)
 3    format(i2,2x,i2,2x,i2,2x,f7.3,2x,e10.3,f7.2,f7.2,2x,
     1e10.3)
	enddo
c
c --  write to output
c
      write(10,*) ' '
      write(10,*) ' vol #     volume(ft3)      area(ft2) '
      write(10,*) ' '
      do j = 1,nvols
      write(10,144)  j,V(j),Area(j)
	enddo
      write(10,*) ' '
c
 144  format(2x,i2,5x,e10.3,10x,f8.4)
c
c --  write them to output
c
      write(10,*) ' '
      write(10,*) 'node press    enthalpy     gms      gsi  '
      do j = 1,nvols
      write(10,13) j,p(j),h(j),Srce(j),Srcex(j)
	enddo
c
      write(10,*) ' '
      write(10,*) ' node  cdens   mixdens     x(qual)   t(degf) '
      do j = 1,nvols
      write(10,13) j,Rho(j),Rhom(j),Xe(j),t(j)
      end do
c
  13  format(i2,1x,f8.2,1x,f8.2,3x,f8.2,1x,f8.2)
c
      write(10,*) ' '
      write(10,*) 'node elevation(ft)   q(btu/s)'
      do j = 1,nvols
      write(10,17) j,Znode(j),Q(j)
      end do
c
17    format(i2,1x,f10.3,5x,f10.3)
c
c
c
c
c************************************************************************c
c*************************** set option *********************************c
c************************************************************************c
c
      if(iset.eq.0) go to 5000
c
      write(11,*) 'set mprt =',mprt
      write(11,*) 'set mprtflg =',mprtflg
c
      write(11,*)'set nvols=',nvols
      write(11,*)'set links=',links
c
      write(11,*) 'set dt=',dt
c
      do 3000 j = 1,nvols
      write(11,*) 'set bnode(j)=',bnode(j)
3000  continue
c
      write(11,*) 'set intn=',intn
      write(11,*) 'set nsgsextn=',nsgsextn
c
      write(11,*) 'set inlinks=',inlinks
      write(11,*) 'set nsgsexte=',nsgsexte
c
c
c --  loop over elemnt quantities and write out to the output file
c
      do 111 j = 1,links
c
      write(11,*) 'set from(',j,')=',from(j)
      write(11,*) 'set tooo(',j,')=',from(j)
      write(11,*) 'set rlink(',j,')=',rlink(j)
      write(11,*) 'set w(',j,')=',w(j)
      write(11,*) 'set Kk(',j,')=',Kk(j)
      write(11,*) 'set Ak(',j,')=',Ak(j)
      write(11,*) 'set Inrt(',j,')=',Inrt(j)
      write(11,*) 'set zdh(',j,')=',zdh(j)
c
111   continue
c
      write(11,*) 'set hbw = ',hbw
      write(11,*) 'set bw = ',bw
c
5001  continue
      do 290 j = 1,nvols
      write(11,*) 'set V(',j,')=',V(j)
      write(11,*) 'set Area(',j,')=',Area(j)
      write(11,*) 'set p(',j,')=',p(j)
      write(11,*) 'set h(',j,')=',h(j)
      write(11,*) 'set Srce(',j,')=',Srce(j)
      write(11,*) 'set Srcex(',j,')=',Srcex(j)
      write(11,*) 'set Rho(',j,')=',Rho(j)
      write(11,*) 'set Rhom(',j,')=',Rhom(j)
      write(11,*) 'set Xe(',j,')=',Xe(j)
      write(11,*) 'set t(',j,')=',t(j)
      write(11,*) 'set Znode(',j,')=',Znode(j)
      write(11,*) 'set Q(',j,')=',Q(j)
290   continue
c
c
5000  continue
c
	return 
	end
c
	subroutine rdfn
	use global
   5  write(*,1000)
 1000 format ( 'Provide name of the input file:')
      read *, fname
      write(*,*)
      inquire (file=fname,exist=fexist)
      if (.not.fexist) then
         write(*, *)  fname,' does not exist'
         write(*, *) 'Do you want to quit (yes or no)?'
         read (*,1001) answer
         if (answer(1:1).eq.'y'.or.answer(1:1).eq.'Y') stop
         go to 5
      endif
      open (9,file=fname,status='old')
 1001 format (a)
c
c
      read(9,*) iset
c
      if(iset.eq.1) open(unit=11,file='cv.set',status='new')
	return
	end
c
	subroutine initzero
	use global
      do i = 1,size
      bnode(i) = 0
      p(i) = 0
      h(i) = 0
      alp(i) = 0
      t(i) = 0
      Rho(i) = 0
	Rhoo(i)=0.0
      Rhom(i) = 0
      Xe(i) = 0
      Q(i) = 0
      Area(i) = 0
      Znode(i) = 0
      V(i) = 0
      DroDh(i) = 0
      DroDp(i) = 0
      ho(i) = 0
      po(i) = 0
      diag(i) = 0
      Srcex(i) = 0
      Srce(i) = 0
c
      from(i) = 0
      tooo(i) = 0
      rlink(i) = 0
      Kk(i) = 0
      Ak(i) = 0
      w(i) = 0
      Inrt(i) = 0
      y(i) = 0
      zdh(i) = 0
      Rhok(i) = 0
	enddo
	return
	end
c
	subroutine readdata
	use global
c
c***********************************************************************
c                     init file to read input data                     *
c***********************************************************************
c
c --- input deck read
c
card 1  = title card
card 2  = mprt,mprtflg
card 3  = # of volumes, # of links
card 4  = time step size
card 5  = # boundary nodes
card 6  = boundary node #, flag value
card 7  = from nd, to nd, vlv pos, loss, link area, mass flow, inertia
card 8  = vol area, vol volume,pressure,enthalpy,source rhs, srce lhs,
c nodal center elevations,q btu/s
card 9 = # flow (link) b.c.'s
card 10 = link b.c. flag, flow b.c. value
card 11 = # of special elements
card 12 =  link #,frm nd #,link #,s/g link #,lhs srce,rhs srce,link #
card 13 = # of metal nodes
card 14 = mtl nds,1-n,fluid nd,5,6,etc,n
card 15 = temperature mtl nd,ua,mc
c
c
c
      read(9,100) title
100   format(a)
      read(9,*) mprt,mprtflg
      read(9,*) nvols,links
      if(nvols.gt.size) write(*,*) 'exceeded maxnodes'
      if(links.gt.size) write(*,*) 'exceeded maxlinks'
      read(9,*) dt
c
c --  read # of boundary nodes, msgnnpb
c --  and read their values
c --  set the boundary nvols
c --  set to 1 for regular node b.c., for both pressure & enthalpy
c --  set to 2 for an enthalpy only b.c. for s/g nodes
c
      read(9,*) msgnnpb
      do k = 1,msgnnpb
      read(9,*) num,bnode(num)
      enddo
c
      do i = 1,links
      read(9,*) k,from(k),tooo(k),rlink(k),Kk(k)
     &,Ak(k),w(k),Inrt(k)
	enddo
c
      do i = 1,nvols
      read(9,*) k,Area(k),V(k),p(k),h(k),
     &Srce(k),Srcex(k),Znode(k),Q(k)
	enddo
c
	return
	end
c
	subroutine elev
	use global
c
c  --  calculate elev changes
c
      do i = 1,links
      nu = from(i)
      nd = tooo(i)
      zdh(i) = Znode(nd) - Znode(nu)
	enddo
c
	return
	end
c
	subroutine chk4zerov
	use global
c
c  --  check for zero volume
c
      do i = 1,nvols
      if(V(i).gt.0.) go to 606
      write(*,*) ' zero volume for node = ',i
      write(10,*) '  zero volume for node = ',i
606   continue
	enddo
	return
	end
c
      subroutine thermo
	use global
	use steam
	implicit none
c
c  this subroutine evaluates the fluid thermodynamic properties
c  of the fluid at each mesh point.
c
c  input:
c    
c        p... fluid pressure (pa)
c        tl.. liquid phase tempurature (k)
c        tg.. vapor phase tempurature (k)
c
c  output:
c
c        ts    ... fluid saturation temp (k)
c        es    ... saturation internal energy (j/kg)
c        hgs   ... saturation enthalpy (j/kg)
c        el    ... liquid internal energy (j/kg)
c        hl    ... liquid enthalpy (j/kg)
c        rol   ... liquid density (kg/m**3)
c        eg    ... vapor internal energy (j/kg)
c        hg    ... vapor enthalpy (j/kg)
c        rog   ... vapor density (kg/m**3)
c        dtsdp ... derivative of ts w/ respest to p
c        desdp ...     "      "  es      "     "  p
c        dhgsdp ..     "      " hsg      "     "  p
c        deldtl ..     "      "  el      "     "  tl
c        droldp ..     "      " rol      "     "  p
c        drldtl ..     "      " rol      "     "  tl
c        degdtg ..     "      "  eg      "     "  tg
c        degdp  ..     "      "  eg      "     "  p
c        drogdp ..     "      " rog      "     "  p
c        drgdtg ..     "      " rog      "     "  tg
c
	a1=c1*c2/c28
	a2=c2-1.0
	a3=-c4*c5/c23
	a4=c5-1
	a5=c45*c49
	a6=2*c45*c48
	a7=4*c44*c45
	a8=3*c43*c45
	a9=2*c42*c45
	a10=c41*c45
	a11=2*c26/(c24+c20)
	a13=a11*(1+c26)
	a12=1/a13
	a14=1/c28
	a15=1/c23
	a16=2*c11
	a17=2*c14
	a18=2*c17
c
c  loop through all mesh points
c
      do 100 i=1,nvols
c
	call pset
c
c  saturation properties
c  
c     temperature:
c
      tsat(i)=c1*(a14*peos(i))**c2 + c3
      dtsdp(i)=a1*(a14*peos(i))**a2  
c
c     internal energy:
c
      if(peos(i).lt.twoe6)go to 10
      es(i)=c12 + (c14*peos(i) + c13)*peos(i)
      desdp(i)=c13 + two*c14*peos(i)
      go to 20
10    es(i)=c6 +c7*(one/(c8+peos(i)))
      desdp(i)=-c7*(one/(c8+peos(i)))**2
20    continue
c
c     enthalpy:
c
      if(peos(i).lt.twoe6)go to 30
      gs=c15 + (c17*peos(i) + c16)*peos(i)
	dgsdp=2*c17*peos(i)+c16
      go to 40
30    gs=c9 + (c11*peos(i) + c10)*peos(i)
	dgsdp=2*c11*peos(i)+c10
40    continue
      hgs(i)=es(i)*gs
      dhgsdp(i)=desdp(i)*gs
      hsg(i)=hg0 + hg1*peos(i) + hg2*peos(i)**2 + hg3*peos(i)**3
     1       + hg4*peos(i)**4
      dhsgdp(i)=hg1 + two*hg2*peos(i) + three*hg3*peos(i)**2
     1       + four*hg4*peos(i)**3
      hsl(i)=hl0 + hl1*peos(i) + hl2*peos(i)**2 + hl3*peos(i)**3
     1       + hl4*peos(i)**4  + hl5*peos(i)**5
      dhsldp(i)=hl1 + two*hl2*peos(i) + three*hl3*peos(i)**2
     1       + four*hl4*peos(i)**3 + five*hl5*peos(i)**4
c
c     heat capacity:
c
      cps=c4*(one-a15*tsat(i))**c5
      dcpsdp=a3*dtsdp(i)*(one-a15*tsat(i))**a4
c
c     calculate mixing cup quality
c
	Xe(i)=(heos(i)-hsl(i))/(hsg(i)-hsl(i))
	if(Xe(i).lt.0.0) Xe(i)=0.0
	if(Xe(i).gt.1.0) Xe(i)=1.0
c
	if(Xe(i).gt.0.0.and.Xe(i).lt.1.0) then
c
c     liquid sat
c
      if(tsat(i).lt.tl0)go to 500
      elsat(i)=sh0 +sh1*tsat(i) + sh2*tsat(i)**2 + sh3*tsat(i)**3
      deldtl(i)=sh1 + two*sh2*tsat(i) + three*sh3*tsat(i)**2
      go to 600
500   elsat(i)=sl0 + sl1*tsat(i) +sl2*tsat(i)**2 + sl3*tsat(i)**3
      deldtl(i)=sl1 + two*sl2*tsat(i) + three*sl3*tsat(i)**2
600   continue
	delsatdp=deldtl(i)*dtsdp(i)
	rolsat(i)=c47 + (c45*elsat(i))*(c48*c45*elsat(i) + c49) + c51*peos(i)
	drldelsat = 2*c45*c45*c48*elsat(i)+c45*c49
	drlsatdp  = drldelsat*delsatdp+c51
c
c     vapor sat
c
c
c  vapor properties
c
      if(peos(i).lt.twoe6)go to 900
      dgsdp=c16 + a18*peos(i)
      go to 950
900   dgsdp=c10 + a16*peos(i)
950   continue
      x=tg(i)-tsat(i)
c
c     density (sat vapor):
c
      yy=(gs-one)*es(i)
      rog(i)=peos(i)/yy
      drgdeg=-rog(i)/yy
      drogdp(i)=rog(i)*(one/peos(i) -(es(i)*dgsdp + (gs-one)*desdp(i)
     +         ))*(one/yy) + drgdeg*desdp(i)
c
c
      nuf=1/rolsat(i)
	nug=1/rog(i)
	dnudh=(nug-nuf)/(hsg(i)-hsl(i))
	dnugdp=-nug*nug*drogdp(i)
	dnufdp=-nuf*nuf*drlsatdp
	dnudp=Xe(i)*(dnugdp-dnudh*dhsgdp(i))+(1-Xe(i))*
     1(dnufdp-dnudh*dhsldp(i))
	Rho(i)=1/(Xe(i)*nug+(1-Xe(i))*nuf)
	DroDp(i)=-Rho(i)*Rho(i)*dnudp
	DroDh(i)=-Rho(i)*Rho(i)*dnudh
c
	endif
c
c  liquid properties
c
c     internal energy:
c
      if(Xe(i).le.0) then
	tl(i)=tsat(i)
	      el(i)=heos(i)
	do k=1,5
      if(tl(i).lt.tl0)go to 50
      deldtl(i)=-(sh1 + two*sh2*tl(i) + three*sh3*tl(i)**2)
	Fbar=el(i)-(sh0 +sh1*tl(i) + sh2*tl(i)**2 + sh3*tl(i)**3)
	dtl=-Fbar/deldtl(i)
	tl(i)=tl(i)+dtl
      go to 60
50    continue
      deldtl(i)=-(sl1 + two*sl2*tl(i) + three*sl3*tl(i)**2)
	Fbar=el(i)-(sl0 + sl1*tl(i) +sl2*tl(i)**2 + sl3*tl(i)**3)
	dtl=-Fbar/deldtl(i)
	tl(i)=tl(i)+dtl
60    continue
	el(i)=sl0 + sl1*tl(i) +sl2*tl(i)**2 + sl3*tl(i)**3
	enddo
c
c     density:
c
      rol(i)=c47 + (c45*el(i))*(c48*c45*el(i) + c49) + c51*peos(i)
      droldp(i)=c51
	droldh(i)=2*c45*el(i)*c48*c45+c45*c49
      drldtl(i)=(a5 + (c45*el(i))*a6)*deldtl(i)
	Rho(i)=rol(i)
	DroDh(i)=droldh(i)
	DroDp(i)=droldp(i)
c
c     enthalpy:
c
      hl(i)=el(i) + (peos(i)/rol(i))
	endif
c
c  vapor properties
c
c     density (superheated vapor):
c
c  vapor properties
c
c     density (superheated vapor):
c
	if(Xe(i).ge.1.0) then
c
	tg(i)=tsat(i)
	do k=1,50
	x=tg(i)-tsat(i)
      xx=(a11*cps - one)
	xx2=xx**2
      betaa=(tsat(i)**2)*(one-(one/xx2))
      xxx=tg(i)**2-betaa
      eg(i)=es(i) + a12*(x + sqrt(xxx)-(tsat(i)/(xx)))
	degdtg(i)=a12*(1+tg(i)/sqrt(xxx))
      dbetdp=(two/tsat(i))*(betaa*dtsdp(i) + ((a11*tsat(i)**3)/(xx**3))
     +       *dcpsdp)
	degdp(i)=desdp(i)-a12*(dtsdp(i)+0.5*dbetdp/sqrt(xxx)+dtsdp(i)/xx
     +-a11*tsat(i)*dcpsdp/xx**2)
      yy=(gs-one)*es(i) + c26*(eg(i)-es(i))
      rog(i)=peos(i)/yy
	drgdeg=-c26*rog(i)/yy
	drgdhg=(1./drgdeg)-peos(i)/rog(i)**2
	drgdhg=1/drgdhg
c
      if(peos(i).lt.twoe6) then
	dgsdp=c10 + a16*peos(i)
	else
      dgsdp=c16 + a18*peos(i)
	endif
c
      drogdp(i)=rog(i)*(one/peos(i) -(es(i)*dgsdp +(gs-one-c26)*desdp(i)
     +         ))*(one/yy) + drgdeg*degdp(i)
      hg(i)=eg(i) + (peos(i)/rog(i))
	Fbar=rog(i)*(heos(i)-eg(i))-peos(i)
      dFbardtg=drgdeg*degdtg(i)*(heos(i)-eg(i)) -rog(i)*degdtg(i)
	dtg=-Fbar/dFbardtg
	tg(i)=dtg+tg(i)
	if(abs(hg(i)-heos(i))/heos(i).lt..1) goto 128
	enddo
128   continue
	Rho(i)=rog(i)
	DroDh(i)=drgdhg
	DroDp(i)=drogdp(i)
      endif
c	drgdhg=drgdeg*degdhg
c      press=peos(i)*convpm2pe
c      hvap=heos(i)*convj2btu/convkg2lbm
c      vsup1=(cnr2(1)+(cnr2(2)+cnr2(3)*hvap)*hvap)/press               
c      vsup2=cnr2(4)+(cnr2(5)+cnr2(6)*hvap)*hvap                     
c      vsup3=cnr2(7)+(cnr2(8)+cnr2(9)*hvap)*hvap*press                        
c      vsup4=cnr2(10)+(cnr2(11)+cnr2(12)*hvap)*hvap*press**2
c
c	dvsup1dp=-(cnr2(1)+(cnr2(2)+cnr2(3)*hvap)*hvap)/press**2
c	dvsup3dp=(cnr2(8)+cnr2(9)*hvap)*hvap
c	dvsup4dp=2*(cnr2(11)+cnr2(12)*hvap)*hvap*press
c
c	dvsup1dh=(cnr2(2)+2*cnr2(3)*hvap)/press
c	dvsup2dh=(cnr2(5)+2*cnr2(6)*hvap)
c	dvsup3dh=(cnr2(8)+2*cnr2(9)*hvap)*press
c	dvsup4dh=(cnr2(11)+2*cnr2(12)*hvap)*press**2     
c                
c      rog(i)=(vsup1+vsup2+vsup3+vsup4)
c	rog(i)=1./rog(i)              

c	Rho(i)=rog(i)/convrom2roe
c	DroDp(i)=-(dvsup1dp+dvsup3dp+dvsup4dp)*rog(i)**2
c	DroDp(i)=DroDp(i)*convpm2pe/convrom2roe
c	DroDh(i)=-(dvsup1dh+dvsup2dh+dvsup3dh+dvsup4dh)*rog(i)**2
c	DroDh(i)=DroDh(i)*convj2btu/(convrom2roe*convkg2lbm)
c      DroDh(m) = DroDh(m)*/
c      DroDp(m) = DroDp(m)*/
c	real convpm2pe,convrom2roe,convj2btu,convkg2lbm  
c
      hg(i)=eg(i) + (peos(i)/rog(i))
c
c 
100   continue
      return
      end 

c
	subroutine pset
	use global
	use steam
	implicit none
	if(units.eq.1) then
	peos(i)=p(i)/convpm2pe
	heos(i)=h(i)*convkg2lbm/convj2btu
	else
	peos(i)=p(i)
	heos(i)=h(i)
	endif
	return
	end
c
      subroutine derivthermo
	use global
	use steam
	units=1
	call thermo
c
c
      do m=1,nvols
      DroDh(m) = DroDh(m)*convrom2roe*convkg2lbm/convj2btu
      DroDp(m) = DroDp(m)*convrom2roe/convpm2pe
c	DroDh(m)=-1.75e-06
c	DroDp(m)=-5e-08
	enddo
      return
      end
c
c	real convpm2pe,convrom2roe,convj2btu,convkg2lbm